/******************************************************************************
*  Arena of battling gladiators (fighters).
*  Meant to assess average abilities & hp at advanced levels. 
*
*  @author   Daniel R. Collins (dcollins@superdan.net)
*  @since    2014-05-20
*  @version  1.0
******************************************************************************/

import java.util.Vector;
import java.util.Collections; 

public class Arena {

	//--------------------------------------------------------------------------
	//  Fields
	//--------------------------------------------------------------------------
	static final int NUM_FIGHTERS = 10000;
	static final int NUM_CYCLES = 1000;
	static final int STATS_TO_LEVEL = 12;
	static final boolean SORT_LIST = false;
	Vector<Persona> fighterList;

	//--------------------------------------------------------------------------
	//  Constructors
	//--------------------------------------------------------------------------

	/**
	*  Constructor (full fields).
	*/
	public Arena () {
		fighterList = new Vector<Persona>();
		for (int i = 0; i < NUM_FIGHTERS; i++) {
			fighterList.add(
				new Persona(null, Alignment.Neutral, ClassID.Fighter));
		}
	}

	//--------------------------------------------------------------------------
	//  Methods
	//--------------------------------------------------------------------------

	/**
	*  Run the arena's top-level algorithm.
	*/
	void runArena () {
		for (int cycle = 0; cycle < NUM_CYCLES; cycle++) {
			organizeList();
			fightPairedDuels();
			bringOutYourDead();
			setAllToMaxHitPoints();
			recruitMoreFighters();
		}
	}

	/**
	*  Organizes the list by either sorting or shuffling.
	*/
	void organizeList () {
		if (SORT_LIST)
			Collections.sort(fighterList, Collections.reverseOrder());
		else
			Collections.shuffle(fighterList);
	}

	/**
	*  Set everyone to max hit points.
	*/
	void setAllToMaxHitPoints () {
		for (Persona p: fighterList) {
			p.setHitPointsToMax();
		}	
	}

	/**
	*  Pair off all fighters & have them duel.
	*/
	void fightPairedDuels () {
		for (int i = 0; i < fighterList.size()-1; i+=2) {
			pairedDuel(fighterList.get(i), fighterList.get(i+1));
		}	
	}

	/**
	*  Remove all dead from the fighter list.
	*/
	void bringOutYourDead () {
		int idx = 0;
		while (idx < fighterList.size()) {
			if (fighterList.get(idx).isDead())
				fighterList.remove(idx);
			else
				idx++;					
		}
	}

	/**
	*  Fill out the list to the full number of fighters.
	*/
	void recruitMoreFighters() {
		while (fighterList.size() < NUM_FIGHTERS) {
			fighterList.add(
				new Persona(null, Alignment.Neutral, ClassID.Fighter));
		}
	}

	/**
	*  Fight one paired duel.
	*  TODO: If somehow circumstances cause two unhittable opponents
	*  or no damage, then this method might go infinite loop.
	*/
	void pairedDuel (Persona p1, Persona p2) {
		Persona winner, loser;

		// Flip coin for initiative
		if (Math.random() > 0.5) {
			Persona swap = p1;
			p1 = p2;
			p2 = swap;
		}	

		// Strike blows
		while (true) {
			p1.strike(p2);
			if (p2.isDead()) {
				winner = p1;
				loser = p2;
				break;
			}
			p2.strike(p1);
			if (p1.isDead()) {
				winner = p2;
				loser = p1;
				break;			
			}
		}
		
		// Award XP to winner
		awardXP(winner, loser);
	}

	/**
	*  Award XP to one fighter for defeating another.
	*  Note: No reduction for difference in levels.
	*/
	void awardXP (Persona winner, Persona loser) {
		int HD = loser.getHitDiceNum();
		int xpAward = totalXPAward(HD);
		winner.addXP(xpAward);
	}

	/**
	*  Compute total XP award (monster + expected treasure).
	*  Treasure accounts for about 95% of XP at 1st level, 
	*  -1% per added level; here we invert that.
	*  See my DeltasDnD blog 2014-02-25.
	*/
	int totalXPAward (int HD) {
		double treasPercent = 0.95 - 0.01 * (HD-1);	
		int totalXP = (int) 
			Math.round(1/(1-treasPercent) * baseXPAward(HD));
		return totalXP;
	}
	
	/**
	*  Compute the base XP award by HD (as per Sup-I).
	*  No partial-HD or special abilities are included here.
	*/
	int baseXPAward (int HD) {
		switch (HD) {
			case 1: return 10;
			case 2: return 20;
			case 3: return 35;
			case 4: return 75;
			case 5: return 175;
			case 6: return 275;
			case 7: return 450;
			case 8: return 650;
			case 9: case 10: return 900;
			case 11: case 12: return 1100;
			case 13: case 14: return 1350;
			case 15: case 16: return 1650;
			case 17: case 18: case 19: case 20: return 2000;
		}
		if (HD > 20)
			return 2500;
		else
			return 0;
	}

	/**
	*  Print all of the fighters (for testng small groups).
	*/
	void printAllFighters() {
		for (Persona p: fighterList) {
			System.out.println(p);
		}	
	}

	/**
	*  Generate & print statistics for all the fighters.
	*/
	public void printStatistics (boolean printStdev) {

		// Create the stats array
		LevelStats stats[] = new LevelStats[STATS_TO_LEVEL];
		for (int i = 0; i < STATS_TO_LEVEL; i++)
			stats[i] = new LevelStats();

		// Add all fighters data into stats array
		for (Persona p: fighterList) {
			int level = p.getHitDiceNum();
			if (0 < level && level < STATS_TO_LEVEL) {
				stats[level-1].addPersona(p);
			}
		}

		// Print out the stats array
		for (int i = 0; i < STATS_TO_LEVEL; i++) {
			System.out.print((i+1) + " ");
			stats[i].printAllStats(printStdev);
		}
		System.out.println();
	}

	/**
	*  Main test method.
	*/
	public static void main (String[] args) {
		Arena a = new Arena();
		a.runArena();
		a.printStatistics(false);
//		a.printAllFighters();
	}
}

