/******************************************************************************
*  Information on one RPG class type (fighter, thief, wizard, etc.).
*
*  @author   Daniel R. Collins (dcollins@superdan.net)
*  @since    2014-05-22
*  @version  1.0
******************************************************************************/

public class ClassInfo {
	//--------------------------------------------------------------------------
	//  Fields
	//--------------------------------------------------------------------------

	/** Class ID/name for this class. */
	ClassID classID;
	
	/** Short name for this class. */
	String shortName;

	/** Experience points to graduate from 1st level. */
	int baseXP;

	/** Maximum number of hit dice. */
	int hitDiceMax;

	/** Type of dice for hit points per level. */
	int hitDiceSides;
	
	/** Additional hit points per level after hitDiceMax. */
	int hitPointAdds;
	
	/** Attack bonus multiplier per level. */
	double atkBonusPerLevel;

	//--------------------------------------------------------------------------
	//  Constructors
	//--------------------------------------------------------------------------

	/**
	*  Constructor (full fields).
	*/
	public ClassInfo (ClassID classID, String shortName, 
			int baseXP, int hitDiceMax, int hitDiceSides, 
			int hitPointAdds, double atkBonusPerLevel) {
		this.classID = classID;
		this.shortName = shortName;
		this.baseXP = baseXP;
		this.hitDiceMax = hitDiceMax;
		this.hitDiceSides = hitDiceSides;
		this.hitPointAdds = hitPointAdds;
		this.atkBonusPerLevel = atkBonusPerLevel;			
	}

	//--------------------------------------------------------------------------
	//  Methods
	//--------------------------------------------------------------------------
	public ClassID getClassID () { return classID; }
	public String getShortName () { return shortName; }
	
	/**
	*  Computes the attack bonus for a given level.
	*/
	public int atkBonusForLevel (int level) {
		return (int)(atkBonusPerLevel * level);	
	}
	
	/**
	*  Returns the XP needed for a given level (approximate).
	*/
	public int XPForLevel (int level) {
		if (level <= 1)
			return 0;
		else if (level <= hitDiceMax) {
			// Starts at baseXP, doubles each level
			return (int) Math.scalb(baseXP, level-2);
		}
		else {
			// After name level, add constant each level			
			int base = (int) Math.scalb(baseXP, hitDiceMax-3);
			return base * (level - hitDiceMax + 2);
		}
	}

	/**
	*  Returns the level for a given XP.
	*/
	public int levelForXP (int XP) {
		int i = 1;
		while (XP >= XPForLevel(i+1)) {
			i++;
		}
		return i;
	}

	/**
	*  Returns hit dice added at a given level (possibly a fixed modifier).
	*/
	public Dice hitDiceForLevel (int level) {
		if (level <= hitDiceMax)
			return new Dice(1, hitDiceSides, 0);
		else
			return new Dice(0, hitDiceSides, hitPointAdds);
	}

	/**
	*  Returns nominal total hit dice for character of this level.
	*  (Note that this would not check minimum 1 for each die.)
	*/
	public Dice hitDiceTotalForLevel (int level) {
		if (level <= hitDiceMax)
			return new Dice(level, hitDiceSides, 0);
		else
			return new Dice(level, hitDiceSides, 
				hitPointAdds * (level-hitDiceMax));
	}

	/**
	*  Returns a string representation of this class info.
	*/
	public String toString() {
		return classID + " Info ("
			+ baseXP + ", " + hitDiceMax + ", " + hitDiceSides + ", "
			+ hitPointAdds + ", " + Math.round(atkBonusPerLevel*10)/10.0 + ")";
	}	

	/**
	*  Main test method.
	*/
	public static void main (String[] args) {
		ClassInfo info1 = new ClassInfo(ClassID.Fighter, 
			"Ftr", 2000, 9, 8, 2, 1.0);
		System.out.println(info1);
		ClassInfo info2 = new ClassInfo(ClassID.Thief, 
			"Thf", 1250, 10, 6, 1, 2.0/3);
		System.out.println(info2);
		ClassInfo info3 = new ClassInfo(ClassID.Wizard, 
			"Wiz", 2500, 11, 4, 1, 0.5);
		System.out.println(info3);
		
// 		for (int i = 1; i <= 20; i++) {
// 			System.out.println("Level " + i + ": " + info1.XPForLevel(i));
// 		}
	}

	/*--------------------------------------------------------------------------
	* NOTES:
	* XP calculations here match OD&D fighters, but vary for thieves & wizards.
	* For more refinement (and spells?), switch internals here to record
	* specifics of each class/level, read in from a text data file.
	* TODO: Armor & weapons allowed, special abilities (feats, skills, spells).
	*-------------------------------------------------------------------------*/
}

