/******************************************************************************
*  Persona (player or non-player character).
*
*  @author   Daniel R. Collins (dcollins@superdan.net)
*  @since    2014-05-20
*  @version  1.0
******************************************************************************/

import java.util.ArrayList;

public class Persona extends Creature {
	//--------------------------------------------------------------------------
	//  Fields
	//--------------------------------------------------------------------------

	/** The six ability scores. */
	int [] abilityScore;

	/** List of class records gained. */
	ArrayList<ClassRecord> classRecordList;

	//--------------------------------------------------------------------------
	//  Constructors
	//--------------------------------------------------------------------------

	/**
	*  Constructor (human, 1st level in some class).
	*/
	public Persona (String name, Alignment alignment, ClassID classID) {
		super(9, 12, 1);

		// Basics
		this.name = name;
		this.race = "Human";
		this.alignment = alignment;
		createRandomAbilityScores();
		classRecordList = new ArrayList<ClassRecord>();
		classRecordList.add(new ClassRecord(classID, getBonus(Ability.Con)));
		setHitPointsToMax();

		// Clunky switch on class (TODO: improve)
		switch (classID) {
			case Fighter:
				moveInches = 9;
				armorClass = 4 - getBonus(Ability.Dex); // chain & shield
				attack = new Attack("Sword", 1 + getBonus(Ability.Str), 
					1,	1, 8, getBonus(Ability.Str));
				break;

			case Thief:
				moveInches = 12;
				armorClass = 7 - getBonus(Ability.Dex); // leather
				attack = new Attack("Sword", 0 + getBonus(Ability.Str), 
					1,	1, 8, getBonus(Ability.Str));
				break;

			case Wizard:
				moveInches = 12;
				armorClass = 9 - getBonus(Ability.Dex); // no armor
				attack = new Attack("Dagger", 0 + getBonus(Ability.Str), 
					1,	1, 4, getBonus(Ability.Str));
				break;
		}
	}

	//--------------------------------------------------------------------------
	//  Methods
	//--------------------------------------------------------------------------

	/**
	*  Creates random ability scores.
	*/
	private void createRandomAbilityScores () {
		abilityScore = new int[Ability.values().length];
		Dice dice = new Dice(3, 6);
 		for (int i = 0; i < abilityScore.length; i++) {
 			abilityScore[i] = dice.roll();
 		}
	}

	/**
	*  Gets an ability score.
	*/
	public int getScore (Ability ability) {
		return abilityScore[ability.ordinal()];	
	}

	/**
	*  Gets the ability score by index.
	*/
	public int getScore (int index) {
		return abilityScore[index];	
	}

	/**
	*  Gets an ability score bonus/modifier.
	*/
	public int getBonus (Ability ability) {
		int score = getScore(ability);
		return bonusFromScore(score);
	}

	/**
	*  Converts an ability score to a bonus (OED-style).
	*/
	private static int bonusFromScore (int score) {
		if (score >= 13) 
			return (score - 10)/3;
		else if (score <= 8) 
			return (score - 11)/3;
		else 
			return 0;
	}

	/**
	*  Returns max level as hit dice number (max over all classes).
	*  TODO: Record in hitDice field at appropriate time. (?)
	*  ## This is probably the clunkiest bit here. Also want level return? ##
	*/
	public int getHitDiceNum () {
		int maxLevel = 0;
		for (ClassRecord record: classRecordList) {
			if (record.getLevel() > maxLevel)
				maxLevel = record.getLevel();
		}
		return maxLevel;
	}

	/**
	*  Returns max hit points (max over all classes).
	*  TODO: Record in maxHitPoints field at appropriate time.
	*/
	public int getMaxHitPoints () {
		int maxHP = 0;
		for (ClassRecord record: classRecordList) {
			if (record.getHitPoints() > maxHP)
				maxHP = record.getHitPoints();		
		}
		return maxHP;
	}

	/**
	*  Returns base attack bonus (max over all classes).
	*/
	public int baseAttackBonus () {
		int maxBAB = 0;
		for (ClassRecord record: classRecordList) {
			if (record.baseAttackBonus() > maxBAB)
				maxBAB = record.baseAttackBonus();
		}
		return maxBAB;
	}

	/**
	*  Returns total XP (sum over all classes).
	*/
	public int totalXP () {
		int sum = 0;
		for (ClassRecord record: classRecordList) {
			sum += record.getXP();
		}
		return sum;
	}

	/**
	*  Strike another persona in melee.
	*  TODO: Arguably this should be in some outside combat manager.
	*  Or at least in the super Creature object.
	*/
	public void strike(Persona target) {
		for (int i = 0; i < attack.getRate(); i++) {
			int naturalRoll = (new Dice(20)).roll();
			int totalRoll = naturalRoll + attack.getBonus() + target.getAC();
			if (totalRoll >= 20 || naturalRoll == 20) {
				int damage = attack.getDamage().roll();
				target.takeDamage(damage);
			}
		}
	}

	/**
	*  Add XP to this persona.
	*  TODO: Currently adds to only the first class.
	*  TODO: A bit clunky that hp in class record, attack here.
	*  Attack bonus only looks at first class.
	*/
	public void addXP (int XP) {
		ClassRecord record = classRecordList.get(0);
		record.addXP(XP, getBonus(Ability.Con));
		Attack newAttack = new Attack(attack.getName(),
			record.baseAttackBonus() + getBonus(Ability.Str),
			1, attack.getDamage());
		attack = newAttack;
	}

	/**
	*  Returns a string representation of this persona.
	*/
	public String toString() {
		return (name == null ? "" : name + ", ")
			+ (race == null ? "" : race + " ")
			+ classString() + " "
			+ "(AC " + getAC() + ", MV " + getMV() + ", HD " + getHD() 
			+ ", hp " + getHP() + ", Atk " + attack + "; " 
			+ abilityString() + "; XP " + totalXP() + ")";
	}	

	/**
	*  String representation of all class & levels.
	*/
	public String classString() {
		String s = "";
		for (ClassRecord record: classRecordList) {
			s += record.getInfo().getShortName() + record.getLevel();
			if (record != classRecordList.get(classRecordList.size()-1))
				s += "/";
		}
		return s;	
	}

	/**
	*  String representation of all ability scores.
	*/
	public String abilityString() {
		String s;
		s = "Str " + abilityScore[0] + ", Int " + abilityScore[1]
			+ ", Wis " + abilityScore[2] + ", Dex " + abilityScore[3]
			+ ", Con " + abilityScore[4] + ", Cha " + abilityScore[5];
		return s;
	}

	/**
	*  Main test method.
	*/
	public static void main (String[] args) {
		Persona p = new Persona(null, Alignment.Neutral, ClassID.Fighter);
		System.out.println(p);
	}
	
	/*--------------------------------------------------------------------------
	* TODO:
	* Class & level(s) with XP, 
	* Equipped armor, shield, weapon.
	* Later: Languages, special abilities, equipment list, encumbrance.
	* Note: Not called "character" to not conflict with java.lang.Character
	* Using Ability enum array saves maintaining 12 individual methods.
	*-------------------------------------------------------------------------*/
}

